function mesa_filter = cortex_get_mesa_filters( rho )
% Get a cell-array of mesa filters used in Watson's cortex transform
%
% mesa_filter = cortex_get_mesa_filters( rho )

band_count = 6;

for band=1:band_count-1
    mesa_filter{band} = get_mesa_filter( rho, 2^-(band-1) );
end
mesa_filter{band_count} = get_base_filter( rho, 2^-(band_count-1) );

end

function F=get_mesa_filter( rho, rho_half )
  FILTER_TRASITION_FREQENCY=0.66666666;
  tw = rho_half * FILTER_TRASITION_FREQENCY;
  F = ones( size( rho ) ) .* (rho < (rho_half - tw/2)) + ...
      1/2 * (1 + cos( pi*(rho - rho_half + tw/2)/tw ) ) .* ...
        ((rho_half - tw/2) < rho) .* (rho < (rho_half + tw/2));
end

function F=get_base_filter( rho, rho_half )
  FILTER_TRASITION_FREQENCY=0.66666666;
  tw = rho_half * FILTER_TRASITION_FREQENCY;
  sigma = 1/3*(rho_half + tw/2);
  F = exp( -rho.^2/(2*sigma^2) ) .* (rho < (rho_half + tw/2) );
end
