function D = create_theta_image( im_size )
% Create matrix that contains angles, given in radians.
%
% D = create_theta_image( im_size )
% im_size     - [height width] vector with image size
%
% Useful for constructing Fourier-domain filters based on CSF data.
%

nyquist_freq = 0.5;
half_size = floor(im_size/2);
odd = mod( im_size, 2 );
freq_step = nyquist_freq./half_size;

if( odd(2) )
    xx = [ linspace( 0, nyquist_freq, half_size(2)+1 ) linspace( -nyquist_freq, -freq_step(2), half_size(2) ) ];
else
    xx = [ linspace( 0, nyquist_freq-freq_step(2), half_size(2) ) linspace( -nyquist_freq, -freq_step(2), half_size(2) ) ];
end

if( odd(1) )
    yy = [ linspace( 0, nyquist_freq, half_size(1)+1 ) linspace( -nyquist_freq, -freq_step(1), half_size(1) ) ];
else
    yy = [ linspace( 0, nyquist_freq-freq_step(1), half_size(1) ) linspace( -nyquist_freq, -freq_step(1), half_size(1) ) ];
end

[XX YY] = meshgrid( xx, yy );

D = atan2( -YY, XX );


end