function S = csf_daly( ro, theta, l_adapt, img_size, viewing_dist )
% Contrast Sensitivity Function from Daly's VDP paper
%
% S = csf_daly( rho, theta, l_adapt, img_size, viewing_dist )
%    rho - spatial frequency in cycles per degree
%    theta - spatial angle in radians
%    l_adapt - luminance of adaptation
%    img_size - image size given in visual degrees^2 
%    viewing_dist - viewing distance in meters (default = 0.5m)
%    S - sensitivity

  P=250; % Peak CSF sensitivity, from Daly's paper
  epsilon=0.9;
  A_l = 0.801*(1+0.7./l_adapt).^(-0.2);
  B_l = 0.3*(1+100./l_adapt).^0.15;
  S = ( (3.23*(ro.^2*img_size).^(-0.3)).^5 + 1).^(-1/5) .* ...
      A_l.*epsilon.*ro.*exp(-(B_l.*epsilon.*ro)).* ...
      sqrt( 1+0.06*exp(B_l.*epsilon.*ro) );

  if( exist( 'viewing_dist', 'var' ) )
      d = viewing_dist;
  else
      d = 0.5; % Distance in meters
  end
  r_a = 0.856 * d^0.14;
  ecc = 0;                      % Eccentrity in visual degrees
  r_e = 1 / (1+0.24*ecc);
  ob = 0.78;
  r_theta = (1-ob)/2 * cos(4*theta) + (1+ob)/2;
  
  ro_prime = ro ./ (r_a * r_e * r_theta);

  S_prime = ( (3.23*(ro_prime.^2*img_size).^(-0.3)).^5 + 1).^(-1/5) .* ...
      A_l.*epsilon.*ro_prime.*exp(-(B_l.*epsilon.*ro_prime)).* ...
      sqrt( 1+0.06*exp(B_l.*epsilon.*ro_prime) );

  S = P*min( S, S_prime );
end
