function map = visualize(mask, fl, fa, cr, hl, map_path, w_dist)

% Configuration
global binary_map;  % Whether maps are modualted w/ probablilties
%global w_dist;  % Weights for each type of dist.
global saturation_space; % space for modulating saturation
%global COLOR_FL; % RGB Color codes for distortions
%global COLOR_FA;
%global COLOR_CR;
%global COLOR_HL;

if nargin < 7
    w_dist = [1 1 1 0];
end


binary_map = 0;
saturation_space = linspace(0,1,50);
COLOR_FL = [0 1 0];
COLOR_FA = [0 0 1];
COLOR_CR = [1 0 0];
COLOR_HL = [1 0 1];

% Pre-processing
w_dist = w_dist ./ max(w_dist(:));
fl = combine_bands(fl);
fa = combine_bands(fa);
cr = combine_bands(cr);
%hl = combine_bands(hl);
fl_lut = color_lut(COLOR_FL);
fa_lut = color_lut(COLOR_FA);
cr_lut = color_lut(COLOR_CR);
%hl_lut = color_lut(COLOR_HL);

% Adjust lightness to avoid desatureation

c = 0.63;
mask = 1 - 1./(1+(mask./logMean(mask)).^c);
mask = (mask * 0.7) + 0.2;


%map = vis(mask, {fl fa cr hl}, {fl_lut fa_lut cr_lut hl_lut}, w_dist);
map = vis(mask, {fl fa cr}, {fl_lut fa_lut cr_lut}, w_dist);
%map = pfs_transform_colorspace('RGB', map, 'XYZ');

% Dump the map ...
pfs_write_rgb(map_path, map); 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VISUALIZATION FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Combine distortion maps from multiple bands
function ret = combine_bands(dm)
ret = 1;
for k=1:length(dm)
    ret = ret .* (1 - dm{k});
end
ret = 1 - ret;

% Create look-up tables for maps
function lut = color_lut(c)
global saturation_space;
[h s v] = rgb2hsv(c);
lut_size = length(saturation_space);
for i=1:lut_size
    lut{i} = hsv2rgb([h saturation_space(i) v]);
end

% Paint the distortin maps with appropriate colors
function ret = paint(map, lut)
global saturation_space;
lut_size = length(saturation_space);

warning_displayed = 0;
[rows cols] = size(map);
for x=1:rows
    for y=1:cols
        p = map(x,y);
        if (p > 1 || p < 0) && ~warning_displayed 
            %warning('Map contains invalid values');
            warning_displayed = 1;
        end
        n1 = min(max(sum(saturation_space < (2*p - 1)), 1), lut_size);  % idx of previous value
        n2 = n1 + 1; % idx of the next value
        ret(x,y,:) = lut{n1};
    end
end


% visualize
function map = vis(mask, dist_map, lut, w)
n_map = length(dist_map);
[rows cols] = size(mask);

% generate color maps
for i=1:n_map
    color_map{i} = paint(dist_map{i}, lut{i});
end


map = ones(rows,cols,3);
% Handle overlapping regions
for x=1:rows
    for y=1:cols
        k = zeros(1,n_map);
        for i=1:n_map
            px = color_map{i}(x,y,:);
            if ~(px(1) == 1 && px(2) == 1 && px(3) == 1)
                k(i) = w(i).*dist_map{i}(x,y);
            end
        end
        % Decide on which map to show at (x,y)
        % Just pick the most probable and most weighted
        idx = find(k == max(k));
        map(x,y,:) = color_map{idx(1)}(x,y,:);
    end
end

% Add bg image
mask = mask.^2.2;
map(:,:,1) = map(:,:,1) .*mask;
map(:,:,2) = map(:,:,2) .*mask;
map(:,:,3) = map(:,:,3) .*mask;


function ret = logMean(I)
I(I < 0.0001) = min(I(I(:) > 0.0001));
[rows cols] = size(I);
ret = exp(sum(log(I(:)))/(rows*cols));
