function map = visualize_2col(mask, fl_high, fl_low, fa_high, fa_low, map_path)

% Configuration

LF_SUFFIX = '_lp';
HF_SUFFIX = '_hp';

global combined;
global mix_colors;
global discrete;

combined = 1;
mix_colors = 1;
discrete = 0;


% Read images
%mask = pfs_read_luminance(mask_path);

% if combined
%    fl_low = pfs_read_luminance([fl_path '_lp.pfs']);
%    fl_high = pfs_read_luminance([fl_path '_hp.pfs']);
%    fa_low = pfs_read_luminance([fa_path '_lp.pfs']);
%    fa_high = pfs_read_luminance([fa_path '_hp.pfs']);
%   
% else
%     fl   = pfs_read_luminance(fl_path);
%     fa   = pfs_read_luminance(fa_path);
% end

%[rows, cols] = size(mask);
%global rows; global cols;

if combined
    fl = 1 - (1-fl_high).*(1-fl_low);
    fa = 1 - (1-fa_high).*(1-fa_low);
end

% Adjust lightness to avoid desatureation
c = 0.63;
mask = 1 - 1./(1+(mask./logMean(mask)).^c);
mask = (mask * 0.7) + 0.2;

%D = fl;
% Preprocess maps
%fl = blur_gaussian(fl,1);
%fa = blur_gaussian(fa,1);

if combined
    map = color_test1(mask, fl, fa);
    pfs_write_rgb(map_path, map);
else
    map = color_test1(mask, fl_low, fa_low);
    pfs_write_rgb([map_path(1:end-4) LF_SUFFIX map_path(end-4:end)], map);
    map = color_test1(mask, fl_high, fa_high);
    pfs_write_rgb([map_path(1:end-4) HF_SUFFIX map_path(end-4:end)], map);
end
    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VISUALIZATION FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function map = color_test1(mask, fl, fa)
[rows, cols] = size(mask);
%global rows; global cols;

global mix_colors;
global discrete;


% Thresholds
RANDOM = 0.5;
PROBABLE = 0.75;
SURE = 0.95;

% Initialize distortion map
map = ones(rows*cols, 3);


% Handle intersection of the two maps
FL_FA = find(fl >= RANDOM & fa >= RANDOM);
FL_intersection = intersect(find(fl >= fa), FL_FA);
FA_intersection = intersect(find(fl < fa), FL_FA);


%map_fl = 0;
%map_fa = 0;
if mix_colors
    FL_FL_FA{1} = intersect(find(fl >= SURE), FL_FA);
    FL_FL_FA{2} = intersect(find(fl >= PROBABLE & fl < SURE), FL_FA);
    FL_FL_FA{3} = intersect(find(fl >= RANDOM & fl < PROBABLE), FL_FA);
    FA_FL_FA{1} = intersect(find(fa >= SURE), FL_FA);
    FA_FL_FA{2} = intersect(find(fa >= PROBABLE & fa < SURE), FL_FA);
    FA_FL_FA{3} = intersect(find(fa >= RANDOM & fa < PROBABLE), FL_FA);
    
    if discrete == 0
        map_fl = color_cont(map, fl, [], FL_FL_FA, {[],[],[]}, RANDOM, PROBABLE, SURE, 2);
        map_fa = color_cont(map, [], fa, {[],[],[]}, FA_FL_FA, RANDOM, PROBABLE, SURE, 3);
    elseif discrete == 1
        map_fl = color_semi_disc(map, fl, [], FL_FL_FA, {[],[],[]}, RANDOM, PROBABLE, SURE, 2);
        map_fa = color_semi_disc(map, [], fa, {[],[],[]}, FA_FL_FA, RANDOM, PROBABLE, SURE, 3);
    %elseif discrete == 2
    %    map_fl = color_disc(map, fl, [], FL_FL_FA, {[],[],[]}, RANDOM, PROBABLE, SURE, 2);
    %    map_fa = color_disc(map, [], fa, {[],[],[]}, FA_FL_FA, RANDOM, PROBABLE, SURE, 3); 
    end
    map = (map_fl + map_fa) / 2;
    fl(FL_FA) = 0; 
    fa(FL_FA) = 0;
else
    fl(FA_intersection) = 0;
    fa(FL_intersection) = 0;
end


% Identify Regions
FL{1} = find(fl >= SURE); 
FL{2} = find(fl >= PROBABLE & fl < SURE);
FL{3} = find(fl >= RANDOM & fl < PROBABLE);

FA{1} = find(fa >= SURE);
FA{2} = find(fa >= PROBABLE & fa < SURE);
FA{3} = find(fa >= RANDOM & fa < PROBABLE);

if discrete == 0
    map = color_cont(map, fl, fa, FL, FA, RANDOM, PROBABLE, SURE);
elseif discrete == 1
    map = color_semi_disc(map, fl, fa, FL, FA, RANDOM, PROBABLE, SURE);
else
    map = color_semi_disc(map, fl, fa, FL, FA, RANDOM, PROBABLE, SURE);
end


%F_4 = find((fa < RANDOM) & (fl < RANDOM));
% NO COLOR
%map(F_4, 1) = 0.8;
%map(F_4, 2) = 0.8;

%map(F_4, 3) = 0.8;


% Contribution form mask
map(:,1) = map(:,1) .* mask(:); 
map(:,2) = map(:,2) .* mask(:); 
map(:,3) = map(:,3) .* mask(:); 
map = reshape(map, rows, cols, 3);



% Continious coloring
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function map = color_cont(map, fl, fa, FL, FA, RANDOM, PROBABLE, SURE, s)
if nargin < 9
    s = 6;
end

if ~mod(s,2);
FL_visible = (union(union(FL{1}, FL{2}), FL{3}));
t = (fl(FL_visible) - RANDOM) / (1 - RANDOM);
map(FL_visible, 1) = t;
map(FL_visible, 2) = 1 - t;
map(FL_visible, 3) = 0;
end

if ~mod(s,3)
FA_visible = (union(union(FA{1}, FA{2}), FA{3}));
t = (fa(FA_visible) - RANDOM) / (1 - RANDOM);
map(FA_visible, 1) = t;
map(FA_visible, 2) = t;
map(FA_visible, 3) = 1-t;
end


% Semi Discrete coloring
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function map = color_semi_disc(map, fl, fa, FL, FA, RANDOM, PROBABLE, SURE, s)
if nargin < 9
    s = 6;
end

if ~mod(s,2);
map(FL{1}, 1) = 1;
map(FL{1}, 2) = 0;
map(FL{1}, 3) = 0;

% RED - GREEN
t = (fl(FL{2}) - PROBABLE) / (SURE - PROBABLE);
map(FL{2}, 1) = t ;
map(FL{2}, 2) = 1 - t;
map(FL{2}, 3) = 0;

% GREEN
map(FL{3}, 1) = 0 ;
map(FL{3}, 2) = 1;
map(FL{3}, 3) = 0;
end

if ~mod(s,3);
% YELLOW
map(FA{1}, 1) = 1;
map(FA{1}, 2) = 1;
map(FA{1}, 3) = 0;

% YELLOW - BLUE
t = (fa(FA{2}) - PROBABLE) / (SURE - PROBABLE);
map(FA{2}, 1) = t ;
map(FA{2}, 2) = t;
map(FA{2}, 3) = 1 - t;

% BLUE
map(FA{3}, 1) = 0 ;
map(FA{3}, 2) = 0;
map(FA{3}, 3) = 1;
end

% Discrete coloring
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function map = color_disc(map, fl, fa, FL, FA, RANDOM, PROBABLE, SURE, s)
if nargin < 9
    s = 6;
end

% Configuration
FL_COLOR = [0 0 1];
FA_COLOR = [1 0 0];

% Generate 4*4 color table
FL_COLOR_HSV = rgb2hsv(FL_COLOR);
FA_COLOR_HSV = rgb2hsv(FA_COLOR);
for x = 1:4     % fa
    for y = 1:4 % fl    
        ct(x,y,:) = (0.5 * (hsv2rgb(FA_COLOR_HSV .* [1 ((4-x)/3) 1]) + hsv2rgb(FL_COLOR_HSV .* [1 ((y-1)/3) 1]))); 
    end
end

%imshow(ct);

% Identify and color various regions
for x = 1:4     % fa
    for y = 1:4 % fl  
        if x == 4
            map(intersect(find(fa==0), FL(y-1)), 1) = ct(x,y,1);
            map(intersect(find(fa==0), FL(y-1)), 2) = ct(x,y,2);
            map(intersect(find(fa==0), FL(y-1)), 3) = ct(x,y,3);
        elseif y ==1
            map(intersect(FA{4-x}, find(fl==0)), 1) = ct(x,y,1);
            map(intersect(FA{4-x}, find(fl==0)), 2) = ct(x,y,2);
            map(intersect(FA{4-x}, find(fl==0)), 3) = ct(x,y,3);            
        else
            map(intersect(FA{4-x}, FL(y-1)), 1) = ct(x,y,1);
            map(intersect(FA{4-x}, FL(y-1)), 2) = ct(x,y,2);
            map(intersect(FA{4-x}, FL(y-1)), 3) = ct(x,y,3);
        end
    end
end

map = reshape(map, rows, cols, 3);
imshow(map);




% VDP coloring (for reference only)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function map = color_vdp(mask, D)
[rows, cols] = size(mask);

% Thresholds
RANDOM = 0.5;
MAYBE = 0.625;
PROBABLE = 0.75;
SURE = 0.95;

% Find indices of the pixels belonging to some region 
D_1 = find(D >= SURE);
D_2 = find(D >= PROBABLE & D < SURE);
D_3 = find(D >= MAYBE & D < PROBABLE);
D_4 = find(D >= RANDOM & D < MAYBE);
D_5 = find(D >= 0.5*RANDOM & D < RANDOM);
D_6 = find(D < 0.5*RANDOM);

% Generate the bmp map 
map = ones(rows*cols, 3);

% PURPLE
map(D_1, 1) = 1;
map(D_1, 2) = 0;
map(D_1, 3) = 0.5 * (D(D_1) - SURE) / (1 - SURE);

% RED
map(D_2, 1) = 1;
map(D_2, 2) = 0;
map(D_2, 3) = 0;

% RED - YELLOW 
map(D_3, 1) = 1;
map(D_3, 2) = 1 - (D(D_3) - MAYBE) / (0.5*(PROBABLE-RANDOM));
map(D_3, 3) = 0;

% LIGHT GREEN - YELLOWISH 
map(D_4, 1) = (D(D_4) - RANDOM) / (0.5*(PROBABLE-RANDOM));
map(D_4, 2) = 1;
map(D_4, 3) = 0;

% GREEN
map(D_5, 1) = 0;
map(D_5, 2) = D(D_5) / (RANDOM);
map(D_5, 3) = 0;

% NO COLOR
map(D_6, 1) = 0.8;
map(D_6, 2) = 0.8;
map(D_6, 3) = 0.8;

% Contribution form mask
map(:,1) = map(:,1) .* mask(:); 
map(:,2) = map(:,2) .* mask(:); 
map(:,3) = map(:,3) .* mask(:); 

map = reshape(map, rows, cols, 3);
%imshow(map);


function ret = logMean(I)
I(I < 0.0001) = min(I(I(:) > 0.0001));
[rows cols] = size(I);
ret = exp(sum(log(I(:)))/(rows*cols));
