function [vis_loss vis_ampl vis_rev] = visvdp( target, mask, options )
%
% [vis_cl vis_ca vis_rc] = visvdp2( target, mask, options )
%
%   vis_loss - loss of visible contrast
%   vis_ampl - amplification of invisible contrast
%   vis_rev  - reversal of visible contrast


% compute visibility / invisibility thresholds

P_vis = 0.95;
P_invis = 0.5;
ps_c = logspace( -1, 1, 2000 );
ps_P = psychometric_func( ps_c, 3 );
rng = (ps_P>0.02 & ps_P<0.98);
        
t_vis = interp1( ps_P(rng), ps_c(rng), P_vis );
t_invis = interp1( ps_P(rng), ps_c(rng), P_invis );


if( any( size(target) ~= size(mask) ) )
    error( 'Mask and target image must be the same size' );
end

options = cat( 2, options, { 'no-contrast-masking' } );

[bands_target] = detection_hdrvdp( target, 'target', options );
[bands_mask] = detection_hdrvdp( mask, 'mask', options );

% For each band 
band_count = 6;
mesa_filter = cortex_get_mesa_filters( create_cycdeg_image( size(target), 1 ) );
dom_filter = cell(1,band_count);
for band=1:band_count-1
    dom_filter{band} = mesa_filter{band} - mesa_filter{band+1};
end
dom_filter{band_count} = mesa_filter{band_count};

P_target_invis = cell( size(bands_target,1)-1, size(bands_target,2) );
P_mask_invis = cell( size(bands_target,1)-1, size(bands_target,2) );
P_target_vis = cell( size(bands_target,1)-1, size(bands_target,2) );
P_mask_vis = cell( size(bands_target,1)-1, size(bands_target,2) );
P_loss = cell( size(bands_target,1)-1, size(bands_target,2) );
P_ampl = cell( size(bands_target,1)-1, size(bands_target,2) );
P_rev = cell( size(bands_target,1)-1, size(bands_target,2) );

for b = 1:size(bands_target,1)
    for o=1:size(bands_target,2)
        if( b == length(bands_target) && o > 1 )
            break; % base band
        end        
        
        P_target_invis{b,o} = (1-psychometric_func( bands_target{b,o}/t_invis, 3 ));
        P_mask_invis{b,o} = (1-psychometric_func( bands_mask{b,o}/t_invis, 3 ));
        P_target_vis{b,o} = psychometric_func( bands_target{b,o}/t_vis, 3, 0.5 );
        P_mask_vis{b,o} = psychometric_func( bands_mask{b,o}/t_vis, 3, 0.5 );
%        P_target_barvis{b,o} = (1 - P_target_invis{b,o} - P_target_vis{b,o} );
%        P_mask_barvis{b,o} = (1 - P_mask_invis{b,o} - P_mask_vis{b,o} );

        P = P_mask_vis{b,o} .* P_target_invis{b,o};
        P_loss{b,o} = bp_filter( P .* sign( bands_mask{b,o} ), dom_filter{b} );

        P = P_mask_invis{b,o} .* P_target_vis{b,o};
        P_ampl{b,o} = bp_filter( P .* sign( bands_mask{b,o} ), dom_filter{b} );
        
        P = P_target_vis{b,o} .* P_mask_vis{b,o} .* ((bands_target{b,o}.*bands_mask{b,o})<0);
        P_rev{b,o} = bp_filter( P .* sign( bands_mask{b,o} ), dom_filter{b} );
        
        if( false )
        P_target_vis{b,o} = psychometric_func( bands_target{b,o}, 3.5 ) .* sign( bands_target{b,o} );
        P_mask_vis{b,o} = psychometric_func( bands_mask{b,o}, 3.5 ) .* sign( bands_mask{b,o} );
        P = (1-abs(P_target_vis{b,o})).^2 .* P_mask_vis{b,o}.^2;
        P_loss{b,o} = bp_filter( P .* sign( bands_mask{b,o} ), dom_filter{b} );
        P = (P_target_vis{b,o}).^2 .* (1-abs(P_mask_vis{b,o})).^2;
        P_ampl{b,o} = bp_filter( P .* sign( bands_mask{b,o} ), dom_filter{b} );
        P = abs( P_target_vis{b,o} .* P_mask_vis{b,o} ) .* ((P_target_vis{b,o}.*P_mask_vis{b,o})<0);
        P_rev{b,o} = bp_filter( P .* sign( bands_mask{b,o} ), dom_filter{b} );
        end
    end
end

% Probability summation
vis_loss = cell(1,2);
vis_ampl = cell(1,2);
vis_rev = cell(1,2);
for k=1:2
    vis_loss{k} = ones( size( target ) );
    vis_ampl{k} = ones( size( target ) );
    vis_rev{k} = ones( size( target ) );
end

% For each band
P_band = cell(1,size(bands_target,1));
for b = 1:size(bands_target,1) 
    P_band{b} = ones( size( target ) );
    for o=1:size(bands_target,2)
        if( b == length(P_target_vis) && o > 1 )
            break; % base band
        end
        freq = 1+(b>3);
        vis_loss{freq} = vis_loss{freq} .* (1.0 - P_loss{b,o});
        vis_ampl{freq} = vis_ampl{freq} .* (1.0 - P_ampl{b,o});
        vis_rev{freq} = vis_rev{freq} .* (1.0 - P_rev{b,o});
%        P_band{b} = P_band{b} .* (1.0 - P{b,o});
    end
end

for k=1:2
    vis_loss{k} = (1 - vis_loss{k});
    vis_ampl{k} = (1 - vis_ampl{k});
    vis_rev{k} = (1 - vis_rev{k});
end

end

function Y = bp_filter( X, filter )
Y = abs(real( ifft2( fft2( X ) .* filter ) ));
end